import { ActionRowBuilder, ApplicationCommandOptionData, ApplicationCommandOptionType, ButtonBuilder, ButtonStyle, CommandInteraction, GuildMember } from "discord.js"
import { CBClient, Challenge } from "../cbclient"
import { SlashCommand, createArgumentsObject } from "./slash"

interface ChallengeArguments {
    opponent: GuildMember
}

export class ChallengeCommand implements SlashCommand {
	name = 'challenge'
    description = 'Challenge a player to a danmaku battle!'
    permission = []
    ownerOnly = false
    guildOnly = true
    //guildID = "739645806100873328" //for testing
    args: ApplicationCommandOptionData[] = [
        {
            name: 'opponent',
            type: ApplicationCommandOptionType.User,
            description: "Who to challenge",
            required: true
        }
    ]

	async execute(int: CommandInteraction) {
        const args = createArgumentsObject<ChallengeArguments>(int.options.data);
        const client = int.client as CBClient;
        
        const player1 = await CBClient.findOrCreatePlayer(int.user.id);
        const player2 = await CBClient.findOrCreatePlayer(args.opponent.id);
        
        if (!player1.slot1 || !player1.slot2 || !player1.slot3) return int.reply({
            content: "You don't have a full team!",
            ephemeral: true
        });
        if (!player2.slot1 || !player2.slot2 || !player2.slot3) return int.reply({
            content: "Your opponent doesn't have a full team!",
            ephemeral: true
        });

        const challenge = new Challenge(player1, player2, await int.reply({
            content: `${args.opponent}, you have been challenged!`,
            fetchReply: true,
            components: [new ActionRowBuilder<ButtonBuilder>()
                .addComponents(
                    new ButtonBuilder()
                        .setCustomId(player1.user_id+player2.user_id)
                        .setLabel('Accept')
                        .setStyle(ButtonStyle.Success)
                )]
        }));
        return client.CHALLENGES.set(challenge.id, challenge);
	}
};
