import { ApplicationCommandOptionData, ApplicationCommandOptionType, CommandInteraction, PermissionFlagsBits, User } from "discord.js"
import { CBClient } from "../cbclient"
import { CHARACTERS } from "../characters"
import { Unit } from "../models/unit"
import { createArgumentsObject, SlashCommand } from "./slash"

interface GrantArguments {
    user: User
    character: number
}

export class GrantCommand implements SlashCommand {
	name = 'grant'
    description = 'Grant someone a character'
    permission = [PermissionFlagsBits.Administrator]
    ownerOnly = false
    guildOnly = false
    //guildID = "739645806100873328" //for testing
    args: ApplicationCommandOptionData[] = [
        {
            name: 'user',
            type: ApplicationCommandOptionType.User,
            description: "Who to grant to",
            required: true
        },
        {
            name: 'character',
            type: ApplicationCommandOptionType.Integer,
            description: "Character ID",
            required: true
        }
    ]

	async execute(int: CommandInteraction) {
        const args = createArgumentsObject<GrantArguments>(int.options.data);

        const character = CHARACTERS.get(args.character);
        if (!character) return int.reply({
            content: `That's not a character`,
            ephemeral: true
        });
        await Unit.create({
            user_id: args.user.id,
            character_id: character.id
        });

        return int.reply({
            content: `${args.user} got **${character.name}**!`
        });
	}
};
