import { ApplicationCommandOptionData, ApplicationCommandOptionType, CommandInteraction, EmbedBuilder, GuildMember, User } from "discord.js"
import { CBClient } from "../cbclient"
import { CHARACTERS } from "../characters"
import { Unit } from "../models/unit"
import { createArgumentsObject, SlashCommand } from "./slash"

interface ListArguments {
    page?: number
    user?: GuildMember
}

export class ListCommand implements SlashCommand {
	name = 'list'
    description = 'List your units'
    permission = []
    ownerOnly = false
    guildOnly = true
    //guildID = "739645806100873328" //for testing
    args: ApplicationCommandOptionData[] = [
        {
            name: 'page',
            type: ApplicationCommandOptionType.Integer,
            description: "The page to view",
            required: false
        },
        {
            name: 'user',
            type: ApplicationCommandOptionType.User,
            description: "The user to view",
            required: false
        }
    ]

	async execute(int: CommandInteraction) {
        const args = createArgumentsObject<ListArguments>(int.options.data);
        if (!args.page) args.page = 1;
        args.page = Math.max(args.page, 1);
        const player = await CBClient.findOrCreatePlayer(args.user?.id || int.user.id);
        const user = args.user?.user || int.user

        const units = await Unit.findAll({ where: { user_id: user.id } });

        let liststr = '';
        for (let i = (args.page-1)*10; i < Math.min(args.page*10, units.length); i++) {
            const character = CHARACTERS.get(units[i].character_id)!;
            liststr += `${units[i].id}: ${character.name}\n`;
        }

        const slot1 = await Unit.findOne({ where: { id: player.slot1 || -1 } });
        const slot2 = await Unit.findOne({ where: { id: player.slot2 || -1 } });
        const slot3 = await Unit.findOne({ where: { id: player.slot3 || -1 } });
        const character1 = CHARACTERS.get(slot1?.character_id || -1);
        const character2 = CHARACTERS.get(slot2?.character_id || -1);
        const character3 = CHARACTERS.get(slot3?.character_id || -1);

        return int.reply({
            embeds: [
                new EmbedBuilder()
                    .setTitle(`[${player.elo}] ${user.username}'s Units (${args.page}/${Math.ceil(units.length/10)})`)
                    .setDescription(liststr || 'Empty...')
                    .addFields(
                        { name: 'Equipped', value: `1: ${character1 ? character1.name + ` (${slot1?.id})` : 'Empty'}\n2: ${character2 ? character2.name + ` (${slot2?.id})` : 'Empty'}\n3: ${character3 ? character3.name + ` (${slot3?.id})` : 'Empty'}` },
                        { name: `${Math.round((Date.now()/1000)-player.start-player.spent)} points`, value: `${Math.round((Date.now()/1000)-player.start-player.spent) > 3600 ? 'You can roll!' : 'Keep saving...'}` }
                    )
            ]
        });
	}
};
